# -*- coding: utf-8 -*-
"""
/***************************************************************************
Plugin for the Postgres Versioning System
-----------------------------------------------------------------------------------------------------------------
begin                : 2010-07-31
copyright          : (C) 2010 by Dr. Horst Duester
email                : horst.duester@kappasys.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
import sys
# Import the PyQt and QGIS libraries
from PyQt4.QtCore import * 
from PyQt4.QtGui import *
from qgis.core import *
from qgis.gui import *
from dbTools import *
from Ui_CommitMessage import CommitMessageDialog
#from pgVersionManagerWindow import ManagerWindow

# Initialize Qt resources from file resources.py
import resources_rc,  traceback
# Import the code for the dialog

from pgVersionLoadDialog import PgVersionLoadDialog
from pgVersionInit import PgVersionInit
from Ui_help import HelpDialog
from Ui_LogView import LogView
from pgVersionTools import PgVersionTools


class PgVersion: 

  def __init__(self, iface):
    # Save reference to the QGIS interface
      self.iface = iface
      self.dlgCommitMessage = CommitMessageDialog()
      self.tools = PgVersionTools()
      self.w = None
      self.vsCheck = None
      
  

  def initGui(self):  

    self.helpDialog = HelpDialog()
    self.LogViewDialog = LogView()

    self.LogViewDialog.progressBar.setRange(0, 100)
    self.LogViewDialog.progressBar.setValue(0)
    
    self.toolBar = self.iface.addToolBar("PG Version")
    self.toolBar.setObjectName("PG Version")
    
    
    self.actionInit = QAction(QIcon(":/plugins/pgversion/icons/pgversion-init.png"), QCoreApplication.translate("PgVersion","Prepare Layer for Versioning"), self.iface.mainWindow())
    self.actionLoad = QAction(QIcon(":/plugins/pgversion/icons/pgversion-load.png"), QCoreApplication.translate("PgVersion","Load Versioned Layer"), self.iface.mainWindow())
    self.actionCommit = QAction(QIcon(":/plugins/pgversion/icons/pgversion-commit.png"), QCoreApplication.translate("PgVersion","Commit Changes"), self.iface.mainWindow())
    self.actionRevert = QAction(QIcon(":/plugins/pgversion/icons/pgversion-revert.png"), QCoreApplication.translate("PgVersion","Revert to HEAD Revision"), self.iface.mainWindow())
    self.actionLogView = QAction(QIcon(":/plugins/pgversion/icons/pgversion-logview.png"), QCoreApplication.translate("PgVersion","Show Logs"), self.iface.mainWindow())
#    self.actionDiff = QAction(QIcon(":/plugins/pgversion/icons/pgversion-diff.png"), QCoreApplication.translate("PgVersion","Show Diffs"), self.iface.mainWindow())
#    self.actionCommit.setEnabled(False)
    self.actionDrop = QAction(QIcon(":/plugins/pgversion/icons/pgversion-drop.png"), QCoreApplication.translate("PgVersion","Drop Versioning from Layer"), self.iface.mainWindow())    
    self.actionHelp = QAction(QIcon(""), QCoreApplication.translate("PgVersion","Help"), self.iface.mainWindow())       
 
#    self.actionList =  [ self.actionInit,self.actionLoad,self.actionDiff,  self.actionCommit,self.actionRevert, self.actionLogView, self.actionDrop, self.actionLogView,  self.actionHelp]         
    self.actionList =  [ self.actionInit,self.actionLoad,self.actionCommit,self.actionRevert, self.actionLogView, self.actionDrop, self.actionLogView,  self.actionHelp]         
 
    self.toolBar.addAction(self.actionInit)
    self.toolBar.addAction(self.actionLoad)
#    self.toolBar.addAction(self.actionDiff)
    self.toolBar.addAction(self.actionCommit)
    self.toolBar.addAction(self.actionRevert)
    self.toolBar.addAction(self.actionLogView)

 # Add the Menubar into the new Database Main Menue starting with QGIS1.7
    try:
        for a in self.actionList:
           self.iface.addPluginToDatabaseMenu("PG Version", a)
    except AttributeError:
# For former QGIS Versions use the old Main Menue
       self.menu = QMenu()
       self.menu.setTitle( "PG Version" )  
       self.menu.addActions(self.actionList)    
       self.menuBar = self.iface.mainWindow().menuBar()
       self.menuBar.addMenu(self.menu)  

    # connect the action to the run method
    QObject.connect(self.actionInit, SIGNAL("triggered()"), self.doInit) 
    QObject.connect(self.actionLoad, SIGNAL("triggered()"), self.doLoad) 
    #QObject.connect(self.actionDiff, SIGNAL("triggered()"), self.doDiff)     
    QObject.connect(self.actionCommit, SIGNAL("triggered()"), self.doCommit) 
    QObject.connect(self.actionRevert, SIGNAL("triggered()"), self.doRevert) 
    QObject.connect(self.actionLogView, SIGNAL("triggered()"), self.doLogView) 
    QObject.connect(self.actionDrop, SIGNAL("triggered()"), self.doDrop) 
    QObject.connect(self.actionHelp, SIGNAL("triggered()"), self.doHelp) 
    
    QObject.connect(self.LogViewDialog, SIGNAL("rollbackLayer(QTreeWidgetItem)"), self.doRollback) 
    QObject.connect(self.LogViewDialog, SIGNAL("checkoutLayer(QTreeWidgetItem)"), self.doCheckout) 




  def unload(self):
        # remove menubar
      try:
          for a in self.actionList:
            self.iface.removePluginDatabaseMenu("PG Version", a)
      except:
          del self.menuBar
      del self.toolBar



      
  def doInit(self):
    canvas = self.iface.mapCanvas()
    currentLayer = canvas.currentLayer()
    
    if currentLayer == None:
      QMessageBox.information(None, '', QCoreApplication.translate('PgVersion','Please select a layer for versioning'))
      return    
    else:
      provider = currentLayer.dataProvider()
      uri = provider.dataSourceUri()    
      myDb = self.tools.layerDB('doInit',uri)
      if not self.tools.checkPGVSRevision(myDb):
        return
      mySchema = QgsDataSourceURI(uri).schema()
      
      if len(mySchema) == 0:
        mySchema = 'public'
      myTable = QgsDataSourceURI(uri).table()

      if not self.tools.versionExists(uri):
        myExtent = canvas.extent()
        answer = QMessageBox.question(None, '', QCoreApplication.translate('PgVersion','Do you want to create the version environment for the table ')+mySchema+'.'+myTable, QCoreApplication.translate('PgVersion','Yes'), QCoreApplication.translate('PgVersion','No'))

        if answer == 0:
          sql = "select * from versions.pgvsinit('"+mySchema+"."+myTable+"')"
          result = myDb.runError(sql)      
          myDb.close()
          
          if len(result) > 1:
            QMessageBox.warning(None, 'Init Error', result)
            return
          else:
            messageText = 'Init was successful!\n\n'
            messageText += 'Please remove the layer '+myTable+' from MapCanvas \n '
            messageText += 'and reload the versioned layer with PG Version->Load versioned Layer\n '
            messageText += 'Please set the user permissions to all tables! '
            QCoreApplication.translate('PgVersion',messageText)
            QMessageBox. information(None, 'Init', messageText)
            canvas.setExtent(myExtent)
            canvas.zoomToPreviousExtent()
    pass

  def doLoad(self): 

# Pruefen, ob alle pgvs Funktionen in der Datenbank eingerichtet sind
    dlg = PgVersionLoadDialog()
    if dlg.exec_() == QDialog.Accepted: 
        try:
          versionTableList= dlg.cmbTables.currentText().split('.')  
          connectionName = dlg.cmbServer.currentText()
          self.loadVersionLayer(connectionName,  versionTableList[0],  versionTableList[1])
        except:  
          QMessageBox.information(None, '', QCoreApplication.translate('PgVersion','Please select a versioned layer'))
    return

  def doRollback(self,  item):
      if item == None:
        QMessageBox.information(None, 'Error',  'Please select a valid revision')
        return

      revision = item.text(0)

      canvas = self.iface.mapCanvas()
      currentLayer = canvas.currentLayer()
    
      if currentLayer == None:
        QMessageBox.information(None, '', QCoreApplication.translate('PgVersion','Please select a versioned layer'))
        return    
      else:
        answer = QMessageBox.question(None, '', QCoreApplication.translate('PgVersion','Are you sure to rollback to revision ')+revision, QCoreApplication.translate('PgVersion','Yes'),  QCoreApplication.translate('PgVersion','No'))
        
        if answer == 0:
            provider = currentLayer.dataProvider()
            uri = provider.dataSourceUri()    
            myDb = self.tools.layerDB('doRollback',uri)
            if not self.tools.checkPGVSRevision(myDb):
              return
            mySchema = QgsDataSourceURI(uri).schema()
          
            if len(mySchema) == 0:
              mySchema = 'public'
            myTable = QgsDataSourceURI(uri).table()
            
            sql = "select * from versions.pgvsrollback('"+mySchema+"."+myTable.replace('_version', '')+"',"+revision+")"
            myDb.run(sql)
            myDb.close()
            self.LogViewDialog.close()
            canvas.refresh()
      pass

  def doCommit(self):
    try:
        canvas = self.iface.mapCanvas()
        theLayer = canvas.currentLayer()
        provider = theLayer.dataProvider()
        uri = provider.dataSourceUri()    
        myDb = self.tools.layerDB('commit',  uri)
        mySchema = QgsDataSourceURI(uri).schema()
        if len(mySchema) == 0:
           mySchema = 'public'
        myTable = QgsDataSourceURI(uri).table()    
    except:
        QMessageBox.information(None, '', QCoreApplication.translate('PgVersion','Please select a versioned layer for committing'))
      
    if theLayer == None:
      QMessageBox.information(None, '', QCoreApplication.translate('PgVersion','Please select a versioned layer for committing'))
      return
    else:
      confRecords = self.tools.confRecords(theLayer)

    if  confRecords == None:
      pass
    elif len(confRecords) == 0:
      pass
    elif len(confRecords) == 1:
        if self.w != None:
            self.w = None
            
      # show the dialog
        self.dlgCommitMessage.show()
        result = self.dlgCommitMessage.exec_()

        if result == QDialog.Accepted:
            try:
              sql = "select * from versions.pgvscommit('"+mySchema+"."+myTable.replace('_version', '')+"','"+self.dlgCommitMessage.textEdit.toPlainText()+"')"
#              QMessageBox.information(None, '', sql)
              myDb.run(sql)
              canvas.refresh()
              QMessageBox.information(None, QCoreApplication.translate('PgVersion','Commit'), QCoreApplication.translate('PgVersion','Commit of your changes was successful'))
              self.tools.setModified(False, theLayer)
            except:
              QMessageBox.information(None, '', QCoreApplication.translate('PgVersion','Please select a versioned layer'))
    else:
        self.w = ManagerWindow(theLayer)
        self.w.show()
        QObject.connect(self.w, SIGNAL("mergeCompleted()"),  self.doCommit)
    myDb.close()
    pass
    
  def doCheckout(self,  item):
      revision = item.text(0)
      self.LogViewDialog.btnClose.setEnabled(False)
      QApplication.setOverrideCursor(Qt.WaitCursor)
      if revision == '-----':
        QMessageBox.information(None, 'Error',  'Please select a valid revision')
        return
      canvas = self.iface.mapCanvas()
      theLayer = self.iface.activeLayer()
      provider = theLayer.dataProvider()
      myCrs = provider.crs()
      uri = provider.dataSourceUri()    
      myDb = self.tools.layerDB('checkout', uri)
      mySchema = QgsDataSourceURI(uri).schema()
      myTable = QgsDataSourceURI(uri).table() 
      myKeyColumn = QgsDataSourceURI(uri).keyColumn()
      numFeat = provider.featureCount()
      
      if len(mySchema) == 0:
          mySchema = 'public'

      sql = "  select "+myKeyColumn+", revision, action "
      sql += "from versions."+mySchema+"_"+myTable+"_log "
      sql += "where revision > "+revision+" "
      sql += "order by revision desc "
      result = myDb.read(sql)
      
# Load the LOG-Table for further usage
      logUri = QgsDataSourceURI()
      logUri.setConnection(QgsDataSourceURI(uri).host(), QgsDataSourceURI(uri).port(), QgsDataSourceURI(uri).database(),QgsDataSourceURI(uri).username() , QgsDataSourceURI(uri).password())
      logUri.setDataSource("versions", mySchema+"_"+myTable+"_log", QgsDataSourceURI(uri).geometryColumn(), "revision > "+revision)
      logUri.setKeyColumn(QgsDataSourceURI(uri).keyColumn())
      logLayer = QgsVectorLayer(logUri.uri(), "dummy", "postgres")
      logProvider = logLayer.dataProvider()
         
      if theLayer.geometryType() == 0:
          myGeometry = "Point"
      elif theLayer.geometryType() == 1:
          myGeometry = "Linestring"
      elif theLayer.geometryType() == 2:
          myGeometry = "Polygon"
      
      revLayer = QgsVectorLayer(myGeometry, myTable.replace('_version', '')+"_rev_"+revision, "memory")
      revLayer.setCrs(myCrs)
      revProvider = revLayer.dataProvider()

      mLogLayer = QgsVectorLayer(myGeometry,  myTable.replace('_version', '')+"_rev_"+revision, "memory")
      mLogLayer.setCrs(myCrs)
      mLogProvider = mLogLayer.dataProvider()
      myFieldList = self.tools.getFieldList(logLayer)
      myFieldDefList = []

      for (k,  attr) in myFieldList.iteritems():
          myFieldDefList.append(attr)
    
      mLogProvider.addAttributes( myFieldDefList)
      
      feat = QgsFeature()
      allAttrs = logProvider.attributeIndexes()
      logProvider.select(allAttrs)
      numFeat = numFeat + logProvider.featureCount()
      self.LogViewDialog.progressBar.setMinimum(0)
      self.LogViewDialog.progressBar.setRange(0, 100)
      self.LogViewDialog.progressBar.setValue(50)
      
      count = 1

      while logProvider.nextFeature(feat):
           mFeat = QgsFeature()
           mFeat.setGeometry(feat.geometry())
           mFeat.setAttributeMap(feat.attributeMap())
           mLogProvider.addFeatures( [mFeat] ) 
           self.LogViewDialog.progressBar.setValue(float(count)/float(numFeat)*100.0)
           count = count + 1
      
      
      mLogLayer.startEditing()
      numFields = mLogProvider.fieldCount()
      for i in range(numFields-1):
        mLogLayer.deleteAttribute(numFields-i)

      mLogLayer.commitChanges()
      QgsMapLayerRegistry.instance().addMapLayer(revLayer)
      

      myFieldList = self.tools.getFieldList(theLayer)
      
      myFieldDefList = []

      for (k,  attr) in myFieldList.iteritems():
          myFieldDefList.append(attr)
    
      revProvider.addAttributes( myFieldDefList)

      myFeatList = []

      feat = QgsFeature()
      allAttrs = provider.attributeIndexes()
    
    # start data retreival: fetch geometry and all attributes for each feature
      provider.select(allAttrs)
      attrs = feat.attributeMap()

      while provider.nextFeature(feat):
         if str(feat.id()) not in result[str(myKeyColumn.toUpper())]:
             revFeat = QgsFeature()
             revFeat.setGeometry(feat.geometry())
             revFeat.setAttributeMap(feat.attributeMap())
             myFeatList.append(revFeat)
             revProvider.addFeatures( [revFeat] ) 
             self.LogViewDialog.progressBar.setValue(float(count)/float(numFeat)*100.0)
             count = count + 1


      feat = QgsFeature()
      allAttrs = mLogProvider.attributeIndexes()
      mLogProvider.select(allAttrs)

      while mLogProvider.nextFeature(feat):
           attrs = feat.attributeMap()
           actionFieldId = len(attrs)-5
           if attrs[actionFieldId].toString() == 'delete':
             revFeat = QgsFeature()
             revFeat.setGeometry(feat.geometry())
             revFeat.setAttributeMap(feat.attributeMap())
             myFeatList.append(revFeat)
             revProvider.addFeatures( [revFeat] ) 
             self.LogViewDialog.progressBar.setValue(float(count)/float(numFeat)*100.0)
             count = count + 1

             
      revLayer.startEditing()
      revLayer.commitChanges()
      self.LogViewDialog.progressBar.setValue(100.0)      
      self.LogViewDialog.btnClose.setEnabled(True)
      QApplication.restoreOverrideCursor()
      QApplication.setOverrideCursor(Qt.ArrowCursor)        
      self.iface.mapCanvas().refresh()
      pass  

  
  def doRevert(self):
      try:
        canvas = self.iface.mapCanvas()
        theLayer = self.iface.activeLayer()
        provider = theLayer.dataProvider()
        uri = provider.dataSourceUri()    
        myDb = self.tools.layerDB('revert', uri)
        mySchema = QgsDataSourceURI(uri).schema()
        myTable = QgsDataSourceURI(uri).table()    
      
        if len(mySchema) == 0:
          mySchema = 'public'
      except:
          QMessageBox.information(None, '', QCoreApplication.translate('PgVersion','Please select a versioned layer for reverting'))
          return
      answer = QMessageBox.question(None, '', QCoreApplication.translate('PgVersion','are you sure to revert to the HEAD revision?'), QCoreApplication.translate('PgVersion','Yes'),  QCoreApplication.translate('PgVersion','No'))

      if answer == 0:
          sql = "select * from versions.pgvsrevert('"+mySchema+"."+myTable.replace('_version', '')+"')"
          result = myDb.read(sql)
                
          if len(result)>1:
             QMessageBox.information(None, '',result)
          else:
             QMessageBox.information(None, QCoreApplication.translate('PgVersion','Message'),QCoreApplication.translate('PgVersion','All changes are set back to the HEAD revision: ')+str(result["PGVSREVERT"][0]))
          self.tools.setModified(False, theLayer)
      canvas.refresh()
      myDb.close()
      pass
      
  def doLogView(self):
    try:
        canvas = self.iface.mapCanvas()
        theLayer = self.iface.activeLayer()
        provider = theLayer.dataProvider()
        uri = provider.dataSourceUri()    
        myDb = self.tools.layerDB('logview', uri)
        mySchema = QgsDataSourceURI(uri).schema()
        myTable = QgsDataSourceURI(uri).table()
     
        if len(mySchema) == 0:
           mySchema = 'public'
               
        sql = "select * from versions.pgvslogview('"+mySchema+"."+myTable.replace('_version', '')+"')"
        result = myDb.read(sql)
        
        logHTML = "<html><head></head><body><Table>"
        
        self.LogViewDialog.treeWidget.clear()
        
        itemList = []
        
        for i in range(len(result["PROJECT"])):
            myItem = QTreeWidgetItem()
            myItem.setText(0, result["REVISION"][i])
            myItem.setText(1, result["DATUM"][i])
            myItem.setText(2, result["PROJECT"][i])
            myItem.setText(3, result["LOGMSG"][i])
            itemList.append(myItem)

        self.LogViewDialog.treeWidget.addTopLevelItems(itemList)

        self.LogViewDialog.show()        
        myDb.close()
        canvas.refresh()
        
    except:
        return
          
        
        pass
    
  def doDiff(self):
      QMessageBox.information(None, 'Message', 'Diff is not implemented yet!')
#    try:
#        canvas = self.iface.mapCanvas()
#        theLayer = canvas.currentLayer()
#        provider = theLayer.dataProvider()
#        uri = provider.dataSourceUri()    
#        myDb = self.tools.layerDB('commit',  uri)
#
#        type='diff'
#        self.w = ManagerWindow(theLayer,  type)
#        self.w.show()
#    except:
#        QMessageBox.information(None, '', QCoreApplication.translate('PgVersion','Please select a layer to show the diffs to HEAD'))

  def doDrop(self): 
      
      try:
          canvas = self.iface.mapCanvas()
          theLayer = self.iface.activeLayer()
          provider = theLayer.dataProvider()
          uri = provider.dataSourceUri()    
          mySchema = QgsDataSourceURI(uri).schema()
          if len(mySchema) == 0:
            mySchema = 'public'
          myTable = QgsDataSourceURI(uri).table()    
    
          if theLayer == None:
            QMessageBox.information(None, '', QCoreApplication.translate('PgVersion','Please select a layer for versioning'))
            return    
          else:
            answer = QMessageBox.question(None, '', QCoreApplication.translate('PgVersion','are you sure to to drop pgvs from the table '+mySchema+"."+myTable.replace('_version', '')+'?'), QCoreApplication.translate('PgVersion','Yes'),  QCoreApplication.translate('PgVersion','No'))
            
            if answer == 0:
                myDb = self.tools.layerDB('doDrop',uri)
                sql = "select * from versions.pgvsdrop('"+mySchema+"."+myTable.replace('_version', '')+"')"
                result = myDb.runError(sql)
                myDb.close()
                
                if len(result)>1:
                  QMessageBox.information(None, '',result)
                else:
                  QgsMapLayerRegistry.instance().removeMapLayer(theLayer.getLayerID())      

      except:
          QMessageBox.information(None, QCoreApplication.translate('PgVersion','Error'), QCoreApplication.translate('PgVersion','Please select a versionied layer for dropping'))
      pass 
        

  def doHelp(self):
      userPluginPath = QFileInfo(QgsApplication.qgisUserDbFilePath()).path()+"/python/plugins/pgversion/docs"  
      helpUrl = QUrl()
      helpUrl.setUrl('file://'+userPluginPath+'/help.html')
      self.helpDialog.webView.load(helpUrl)
      self.helpDialog.show()
      pass  

  def loadVersionLayer(self, connectionName,  schema,  table):
        QApplication.setOverrideCursor(Qt.WaitCursor)
        settings = QSettings()
        mySettings = "/PostgreSQL/connections/"+connectionName
        DBNAME = settings.value(mySettings+"/database").toString()
        DBUSER = settings.value(mySettings+"/username").toString()
        DBHOST = settings.value(mySettings+"/host").toString()
        DBPORT = settings.value(mySettings+"/port").toString()
        DBPASSWD = settings.value(mySettings+"/password").toString()
        DBTYPE = 'pg'

        try:
           myDb = DbObj(pluginname=connectionName,typ=DBTYPE,hostname=DBHOST,port=DBPORT,dbname=DBNAME,username=DBUSER, passwort=DBPASSWD)
        except:
           QMessageBox.information(None, QCoreApplication.translate('PgVersion','Error'), QCoreApplication.translate('PgVersion','No Database Connection Established.'))
           return

  
        sql =   "select * from versions.version_tables "
        sql += "where version_table_schema = '"+schema+"' and version_table_name = '"+table+"'"
        layer = myDb.read(sql)    
    
        uri = QgsDataSourceURI()
    
        # set host name, port, database name, username and password
        uri.setConnection(DBHOST, DBPORT, DBNAME, DBUSER, DBPASSWD)    
    
        uri.setDataSource(layer["VERSION_VIEW_SCHEMA"][0], layer["VERSION_VIEW_NAME"][0], ""+layer["VERSION_VIEW_GEOMETRY_COLUMN"][0]+"", "",  layer["VERSION_VIEW_PKEY"][0])
    
        layerName = layer["VERSION_TABLE_NAME"][0]
        vLayer = QgsVectorLayer(uri.uri(), layerName, "postgres")
        
        self.tools.setModified(False, vLayer)
        
        if self.tools.vectorLayerExists(vLayer.name()) or self.tools.vectorLayerExists(vLayer.name()+" (modified)"):
          QMessageBox.warning(None, '', QCoreApplication.translate('PgVersion','Layer '+table+' is already loaded'))
          QApplication.restoreOverrideCursor()
          QApplication.setOverrideCursor(Qt.ArrowCursor)      
          return      
    
        QgsMapLayerRegistry.instance().addMapLayer(vLayer)      
        QObject.connect(vLayer, SIGNAL("layerModified(bool)"), lambda onlyGeometry,  myLayer=vLayer: self.tools.setModified(onlyGeometry, myLayer))
        myDb.close()
        QApplication.restoreOverrideCursor()
        QApplication.setOverrideCursor(Qt.ArrowCursor)        
    
  def doMessage(self):
      QMessageBox.information(None, '', 'Hallo')    
