from qgis.gui import *
from qgis.core import *
from PyQt4.QtGui import *
from PyQt4.QtCore import *
from pgVersionTools import PgVersionTools
import resources_rc,  sys

class ConflictWindow(QMainWindow):
  def __init__(self, layer,  type='conflict'):
    QMainWindow.__init__(self)

    self.tools = PgVersionTools()    
    self.layer = layer
    
    settings = QSettings()
          
    # Restore Window Settings
    settings = QSettings()
    self.restoreGeometry(settings.value("/pgVersion/geometry").toByteArray())
    self.restoreState(settings.value("/pgVersion/windowState").toByteArray())
    
    settings = QSettings()
    self.canvas = QgsMapCanvas()
    self.canvas.setCanvasColor(Qt.white)
    self.canvas.enableAntiAliasing( settings.value( "/qgis/enable_anti_aliasing", QVariant(False) ).toBool() )
    self.canvas.useImageToRender( settings.value( "/qgis/use_qimage_to_render", QVariant(False) ).toBool() )
    action = settings.value( "/qgis/wheel_action", QVariant(0) ).toInt()[0]
    zoomFactor = settings.value( "/qgis/zoom_factor", QVariant(2) ).toDouble()[0]
    self.canvas.setWheelAction( QgsMapCanvas.WheelAction(action), zoomFactor )

    self.setCentralWidget(self.canvas)

    actionZoomIn = QAction(QIcon(":/plugins/pgversion/icons/ZoomIn.png"), QString("Zoom in"), self)
    actionZoomOut = QAction(QIcon(":/plugins/pgversion/icons/ZoomOut.png"),QString("Zoom out"), self)
    actionZoomFull = QAction(QIcon(":/plugins/pgversion/icons/ZoomFullExtent.png"),QString("Fullextent"), self)
    actionPan = QAction(QIcon(":/plugins/pgversion/icons/Pan.png"),QString("Pan"), self)

    actionZoomIn.setCheckable(True)
    actionZoomOut.setCheckable(True)
    actionPan.setCheckable(True)

    self.connect(actionZoomIn, SIGNAL("triggered()"), self.zoomIn)
    self.connect(actionZoomOut, SIGNAL("triggered()"), self.zoomOut)
    self.connect(actionZoomFull, SIGNAL("triggered()"), self.zoomFull)
    self.connect(actionPan, SIGNAL("triggered()"), self.pan)


    self.toolbar = self.addToolBar("Canvas actions")
    self.toolbar.addAction(actionPan)
    self.toolbar.addAction(actionZoomIn)
    self.toolbar.addAction(actionZoomOut)
    self.toolbar.addAction(actionZoomFull)

    
    self.dockWidget = QDockWidget()
    self.dockWidget.setWidget(QTableWidget())
    self.addDockWidget(Qt.BottomDockWidgetArea, self.dockWidget)
    self.tabView = self.dockWidget.widget()

    # create the map tools
    self.toolPan = QgsMapToolPan(self.canvas)
    self.toolPan.setAction(actionPan)
    self.toolZoomIn = QgsMapToolZoom(self.canvas, False) # false = in
    self.toolZoomIn.setAction(actionZoomIn)
    self.toolZoomOut = QgsMapToolZoom(self.canvas, True) # true = out
    self.toolZoomOut.setAction(actionZoomOut)

    self.pan()

    if type=='conflict':
      self.cmbMerge = QComboBox()
      self.cmbMerge.addItems(self.tools.confRecords(self.layer))
    
      btnMerge = QPushButton()
      QObject.connect(self.cmbMerge, SIGNAL("currentIndexChanged()"), self.toggleBtnMerge)
      btnMerge.setText(QCoreApplication.translate('ManagerWindow','solve conflict'))
      self.toolbar.addWidget(self.cmbMerge)
      self.toolbar.addWidget(btnMerge)
      self.manageConflicts()

      QObject.connect(btnMerge, SIGNAL("clicked()"), self.runMerge) 
    elif type=='diff':
      self.loadLayer(self.layer)



    QObject.connect(self.tabView, SIGNAL("itemSelectionChanged()"),  self.showConfObject)    
    self.rBand = QgsRubberBand(self.canvas, False)

  def toggleBtnMerge(self):
      if self.btnMerge.isEnabled():
          self.btnMerge.setEnabled(False)
      else:
          self.btnMerge.setEnabled(True)
      return

  def zoomIn(self):
    self.canvas.setMapTool(self.toolZoomIn)

  def zoomOut(self):
    self.canvas.setMapTool(self.toolZoomOut)

  def zoomFull(self):
      self.canvas.zoomToFullExtent()

  def pan(self):
    self.canvas.setMapTool(self.toolPan)
    

  def closeEvent(self, e):
      """ save window state """
      settings = QSettings()
      settings.setValue("/pgVersion/windowState", QVariant(self.saveState()))
      settings.setValue("/pgVersion/geometry", QVariant(self.saveGeometry()))
      QMainWindow.closeEvent(self, e)    

  def showDiffs(self):
      pass

  def showConfObject(self):
      try:
        myProject =  self.tabView.item(self.tabView.currentRow(), 0).text()
        myId =  self.tabView.item(self.tabView.currentRow(), 3).text()
      except: 
        return
      
      confLayer = self.tools.conflictLayer(self.layer)
      
      try:
        self.rBand.reset()      
        provider = confLayer.dataProvider()
        feat = QgsFeature()
        allAttrs = provider.attributeIndexes()
        provider.select(allAttrs)

        self.rBand.setColor(QColor(0, 0, 255))
        self.rBand.setWidth(5)
      
        while provider.nextFeature(feat):
           geom = feat.geometry()
           attrs = feat.attributeMap()
         
           if attrs[0].toString() == myId and attrs[len(attrs)-4].toString() == myProject:         
             self.rBand.addGeometry(geom,  None)
      except:
        return
      
  def manageConflicts(self):
      vLayer = self.tools.conflictLayer(self.layer) 
      if vLayer == None:
        return
        
      QgsMapLayerRegistry.instance().addMapLayer(vLayer,  False)
      self.vLayerId = vLayer.getLayerID()
      self.canvas.setLayerSet( []  )
      self.canvas.setLayerSet( [QgsMapCanvasLayer(vLayer,  True) , QgsMapCanvasLayer(self.layer,  True)  ]  )
      self.canvas.setExtent(vLayer.extent())
      self.canvas.refresh()
    
      tabData = self.tools.tableRecords(self.layer)

      self.tools.createGridView(self.tabView, tabData[0], tabData[1], 100, 10)


  def runMerge(self): 
        currentLayer = self.layer
        object = self.cmbMerge.currentText()
        
        if currentLayer == None:
            QMessageBox.information(None, '', QCoreApplication.translate('ManagerWindow','Please select a versioned layer for committing'))
            return
        else:
            provider = currentLayer.dataProvider()
            uri = provider.dataSourceUri()    
            myDb = self.tools.layerDB('Merge', uri)
            mySchema = QgsDataSourceURI(uri).schema()
            if len(mySchema) == 0:
                mySchema = 'public'
            myTable = QgsDataSourceURI(uri).table().remove("_version")     
      
        objectArray = object.split(" - ")
        if len(objectArray)==1:
            return
        objId = objectArray[0]
        projectName = objectArray[1]
        
        
        sql ="select versions.pgvsmerge('"+mySchema+"."+myTable.remove("_version")+"',"+str(objId)+",'"+projectName+"')"
        myDb.run(sql)
        
        QgsMapLayerRegistry.instance().removeMapLayer(self.vLayerId)
        self.canvas.refresh()
        
        sql = "select * from versions.pgvscommit('"+mySchema+"."+myTable.remove("_version")+"')"
        result = myDb.run(sql)
        self.cmbMerge.clear()
        self.cmbMerge.addItems(self.tools.confRecords(self.layer))
                
        if len(self.tools.confRecords(self.layer)) ==1:
            self.tabView.clear()
            self.tools.setModified(False, self.layer)
            self.emit(SIGNAL("mergeCompleted()"))
        else:
            tabData = self.tools.tableRecords(self.layer)
            self.tools.createGridView(self.tabView, tabData[0], tabData[1], 100, 10)
            self.manageConflicts()


  def loadLayer(self, layer):
      provider = layer.dataProvider()
      uri = provider.dataSourceUri()    
      mySchema = QgsDataSourceURI(uri).schema()
      myHost = QgsDataSourceURI(uri).host()
      myDatabase = QgsDataSourceURI(uri).database()
      myPassword = QgsDataSourceURI(uri).password()
      myPort = QgsDataSourceURI(uri).port()
      myUsername = QgsDataSourceURI(uri).username()
      myGeometryColumn = QgsDataSourceURI(uri).geometryColumn()
      
      if len(mySchema) == 0:
          mySchema = 'public'
      myTable = QgsDataSourceURI(uri).table().remove("_version")     
      
      myUri = QgsDataSourceURI()
      myUri.setConnection(myHost, myPort, myDatabase, myUsername, myPassword)
      myUri.setDataSource(mySchema, myTable, myGeometryColumn)
      
      myLayer = QgsVectorLayer(myUri.uri(), "", "postgres")
      
      QgsMapLayerRegistry.instance().addMapLayer(myLayer, False)
      self.canvas.setExtent(myLayer.extent())
      self.canvas.setLayerSet([QgsMapCanvasLayer(myLayer)])
      self.canvas.refresh()


  def showGagaObject(self, layer):
      try:
        myProject =  self.tabView.item(self.tabView.currentRow(), 0).text()
        myId =  self.tabView.item(self.tabView.currentRow(), 3).text()
      except: 
        return
      
      confLayer = self.tools.conflictLayer(layer)
      self.rBand.reset()      
      provider = confLayer.dataProvider()
      feat = QgsFeature()
      allAttrs = provider.attributeIndexes()
      provider.select(allAttrs)

      self.rBand.setColor(QColor(0, 0, 255))
      self.rBand.setWidth(5)
      
      while provider.nextFeature(feat):
         geom = feat.geometry()
         attrs = feat.attributeMap()
         
         if attrs[0].toString() == myId and attrs[len(attrs)-4].toString() == myProject:         
           self.rBand.addGeometry(geom,  None)
    
