# -*- coding: utf-8 -*-
"""
/***************************************************************************
Plugin for the Postgres Versioning System
-----------------------------------------------------------------------------------------------------------------
begin                : 2010-07-31
copyright          : (C) 2010 by Dr. Horst Duester
email                : horst.duester@kappasys.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from dbTools import *
from pgVersionTools import  *
from Ui_Ui_pgLoadVersion import Ui_pgLoadVersion

class PgVersionLoadDialog(QDialog, Ui_pgLoadVersion):
  def __init__(self): 
    QDialog.__init__(self)
    self.setupUi(self)
    self.getDbSettings()
    self.tools = PgVersionTools()
    QObject.connect(self, SIGNAL("currentDBChanged(QString)"), self.initDB)   

    
  def getDbSettings(self):
    settings = QSettings()
    settings.beginGroup("PostgreSQL/connections")
    self.cmbServer.addItem('------------')
    self.cmbServer.addItems(settings.childGroups())
    settings.endGroup()
    selectedDb = settings.value("PostgreSQL/connections/selected").toString()
    cmbIndex = self.cmbServer.findText(selectedDb, Qt.MatchExactly)
    self.cmbServer.setCurrentIndex(cmbIndex)    
    
    
  def initDB(self,  selectedServer):    
      if self.cmbServer.currentIndex() == 0:
          return
      settings = QSettings()
      mySettings = "/PostgreSQL/connections/"+selectedServer
      DBNAME = settings.value(mySettings+"/database").toString()
      DBUSER = settings.value(mySettings+"/username").toString()
      DBHOST = settings.value(mySettings+"/host").toString()
      DBPORT = settings.value(mySettings+"/port").toString()
      DBPASSWD = settings.value(mySettings+"/password").toString()
      DBTYPE = 'pg'

      self.lblDBUser.setText(DBUSER)

      try:
          myDb = DbObj(pluginname=selectedServer,typ=DBTYPE,hostname=DBHOST,port=DBPORT,dbname=DBNAME,username=DBUSER, passwort=DBPASSWD)
      except:
          QMessageBox.information(None, QCoreApplication.translate('PgVersionLoadDialog','Error'), QCoreApplication.translate('PgVersionLoadDialog','No Database Connection Established.'))
          return

      if not self.tools.checkPGVSRevision(myDb):
          self.cmbServer.setCurrentIndex(0)
          return
            
      query = "select count(oid) from pg_catalog.pg_proc where proname in ('pgvsinit','pgvscommit','pgvsdrop','pgvsmerge')" 
      sysResult  = myDb.read(query)
 
      query =   "select version_table_schema as schema, version_table_name as table "
      query += "from versions.version_tables "
      query += "order by version_table_schema,version_table_name"

      result = myDb.read(query)
      self.cmbTables.clear()

      try:
          if sysResult["COUNT"][0] == "4":
              if len(result["TABLE"]) > 0: 
                self.cmbTables.addItem("-------------")
                for i in range(len(result["TABLE"])):
                  self.cmbTables.addItem(result["SCHEMA"][i]+'.'+result["TABLE"][i])
          else:
            self.cmbTables.addItem(QCoreApplication.translate('PgVersionLoadDialog','pgvs not installed in the database'))      
      except:
          QMessageBox.information(None, QCoreApplication.translate('PgVersionLoadDialog','Error'), QCoreApplication.translate('PgVersionLoadDialog','No database connection established.\n Please define a valid database connection and try again.'))
          self.close()
          return            
    
#@brief Set default server name
#@param String Server Name
#@return  none
#@author Dr. Horst Duester
#@date 16. March 2009
#@version 1.0
#@todo     
  def setDBServer(self, dbServerName):
     settings = QSettings()
     settings.setValue("PostgreSQL/connections/selected", QVariant(dbServerName))  
     pass    
    
  @pyqtSlot(int)      
  def on_cmbServer_currentIndexChanged(self):
      self.setDBServer(self.cmbServer.currentText())
      self.emit(SIGNAL("currentDBChanged(QString)"),  self.cmbServer.currentText())    
    



    
    
    
    
    

