# -*- coding: utf-8 -*-
#-----------------------------------------------------------
# 
# Rectangles, Ovals, Diamonds
# Copyright (C) 2011 - 2012 Pavol Kapusta <pavol.kapusta@gmail.com>
# Copyright (C) 2011 Benjamin Bohard <benjamin.bohard@orange.fr>
#
# Code adapted from 'fTools Plugin', Copyright (C) 2009  Carson Farmer
#-----------------------------------------------------------
# 
# licensed under the terms of GNU GPL 2
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
# 
#---------------------------------------------------------------------

import math
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from qgis.gui import *
from ui_frmRectOvalDiams import Ui_Dialog


class RectOvalDiams( QDialog, Ui_Dialog ):
    def __init__( self, iface ):
        QDialog.__init__( self )
        self.iface = iface
        self.setupUi( self )
        QObject.connect( self.btnBrowse, SIGNAL( "clicked()" ), self.outFile )
        QObject.connect( self.inShapeBox, SIGNAL( "currentIndexChanged(QString)" ), self.checkLayer )
        QObject.connect( self.inShapeBox, SIGNAL( "currentIndexChanged(QString)" ), self.wupdate )
        QObject.connect( self.inShapeBox, SIGNAL( "currentIndexChanged(QString)" ), self.hupdate )
        QObject.connect( self.inShapeBox, SIGNAL( "currentIndexChanged(QString)" ), self.rupdate )
        QObject.connect( self.rectButton, SIGNAL( "clicked()" ), self.manageGui )
        QObject.connect( self.ovalButton, SIGNAL( "clicked()" ), self.manageGui )
        QObject.connect( self.diamondButton, SIGNAL( "clicked()" ), self.manageGui )
        QObject.connect( self.chBoxRot, SIGNAL( "clicked()" ), self.manageGui )
        self.manageGui()
        self.populateLayers()
        self.wBox.setEnabled(True)
        self.hBox.setEnabled(True)
        self.segLabel.hide()
        self.segspinBox.hide()
        self.rLabel.hide()
        self.rBox.hide()
        self.progressBar.setValue(0)
        self.shape = 1
        self.rotate = False


    def checkLayer( self ):
        inputLayer = unicode( self.inShapeBox.currentText() )
        if inputLayer != "":
            changedLayer = getVectorLayerByName( inputLayer )
      
    def wupdate( self ):
        self.wBox.clear()
        inputLayer = unicode( self.inShapeBox.currentText() )
        if inputLayer != "":
            changedLayer = getVectorLayerByName( inputLayer )
            changedField = changedLayer.dataProvider().fields()
            for i in changedField:
                self.wBox.addItem( unicode( changedField[i].name() ) )

    def hupdate( self ):
        self.hBox.clear()
        inputLayer = unicode( self.inShapeBox.currentText() )
        if inputLayer != "":
            changedLayer = getVectorLayerByName( inputLayer )
            changedField = changedLayer.dataProvider().fields()
            for i in changedField:
                self.hBox.addItem( unicode( changedField[i].name() ) )
        
    def rupdate( self ):
        self.rBox.clear()
        inputLayer = unicode( self.inShapeBox.currentText() )
        if inputLayer != "":
            changedLayer = getVectorLayerByName( inputLayer )
            changedField = changedLayer.dataProvider().fields()
            for i in changedField:
                self.rBox.addItem( unicode( changedField[i].name() ) )
 
    def manageGui( self ):
        if self.chBoxRot.isChecked():
            self.rLabel.show()
            self.rBox.show()
            self.rBox.setEnabled(True)
            self.rotate = True
        else:
            self.rBox.setEnabled(False)
            self.rLabel.hide()
            self.rBox.hide()            
            self.rotate = False
        if self.ovalButton.isChecked():
            self.segLabel.show()
            self.segspinBox.show()
            self.shape = 3
        else:
            self.segLabel.hide()
            self.segspinBox.hide()
        if self.rectButton.isChecked():
            self.shape = 1
        elif self.diamondButton.isChecked():
            self.shape = 2

  
    def populateLayers( self ):
        myList = []
        self.inShapeBox.clear()
        myList = getLayerNames( [ QGis.Point ] )
        self.inShapeBox.addItems( myList )
   
    
    def accept( self ):
        inputLayer = unicode( self.inShapeBox.currentText() )  
        layer = getVectorLayerByName( inputLayer )
        width = unicode( self.wBox.currentText() )
        height = unicode( self.hBox.currentText() )
        rotation = unicode( self.rBox.currentText() )
        if self.inShapeBox.currentText() == "":
            QMessageBox.warning( self, "Rectangles, Ovals, Diamonds", self.tr( "Please specify an input layer" ) )
        elif self.outShape.text() == "":
            QMessageBox.warning( self, "Rectangles, Ovals, Diamonds", self.tr( "Please specify output shapefile" ) )
        elif self.wBox.currentText() == "" or getFieldType( layer, width) == "String":
            QMessageBox.warning( self, "Rectangles, Ovals, Diamonds", self.tr( "Width: Please select numeric field" ) )
        elif self.hBox.currentText() == "" or getFieldType( layer, height) == "String":
            QMessageBox.warning( self, "Rectangles, Ovals, Diamonds", self.tr( "Height: Please select numeric field" ) )
        elif self.rotate == True and (self.rBox.currentText() == "" or getFieldType( layer, rotation) == "String"):
            QMessageBox.warning( self, "Rectangles, Ovals, Diamonds", self.tr( "Rotation: Please select numeric field" ) )
        else:
            self.outShape.clear()
            self.rectovaldiams( self.inShapeBox.currentText(), unicode( self.wBox.currentText() ), unicode( self.hBox.currentText() ), unicode( self.rBox.currentText() ))
            out_text = "\n"
            end_text = self.tr( "\nWould you like to add the new layer to the TOC?" )
            addToTOC = QMessageBox.question( self, "Rectangles, Ovals, Diamonds", self.tr( "Created output shapefile:" ) + "\n"
                                          + unicode( self.shapefileName )
                                          + out_text + end_text, QMessageBox.Yes, QMessageBox.No, QMessageBox.NoButton )
            if addToTOC == QMessageBox.Yes:
                addShapeToCanvas( unicode( self.shapefileName ) )


    def outFile( self ):
        self.outShape.clear()
        ( self.shapefileName, self.encoding ) = saveDialog( self )
        if self.shapefileName is None or self.encoding is None:
            return
        self.outShape.setText( QString( self.shapefileName ) )

    def rectovaldiams( self, ilayer, width, height, rotation):
        #Kudos to Benjamin Bohard for list comprehensions
        layer = getVectorLayerByName( ilayer )
        provider = layer.dataProvider()
        self.progressBar.setRange(0, provider.featureCount())
        index1 = provider.fieldNameIndex( width )
        index2 = provider.fieldNameIndex( height )
        index3 = provider.fieldNameIndex( rotation )
        segments = self.segspinBox.value()
        provider.select(layer.pendingAllAttributesList(), QgsRectangle(), True, True)
        in_feat = QgsFeature()
        out_feat = QgsFeature()
        inGeom = QgsGeometry()
        outGeom = QgsGeometry()
        n = 0
        writer = QgsVectorFileWriter( self.shapefileName, self.encoding, provider.fields(),QGis.WKBPolygon, provider.crs() )
        if self.rotate:
            if self.shape == 1:  
                while(provider.nextFeature(in_feat)):
                    atMap = in_feat.attributeMap()
                    width = atMap[index1].toDouble()[0]
                    height = atMap[index2].toDouble()[0]
                    angle = atMap[index3].toDouble()[0]      
                    xOffset = float(width/2)
                    yOffset = float(height/2)
                    phi = angle*math.pi/180
                    inGeom = in_feat.geometry()
                    x = inGeom.asPoint().x()
                    y = inGeom.asPoint().y()
                    pt1 = (-xOffset, -yOffset)
                    pt2 = (-xOffset, yOffset)
                    pt3 = (xOffset, yOffset)
                    pt4 = (xOffset, -yOffset)
                    points = [pt1, pt2, pt3, pt4]
                    polygon = [[QgsPoint(i[0]*math.cos(phi)+i[1]*math.sin(phi)+x,
                                    -i[0]*math.sin(phi)+i[1]*math.cos(phi)+y) for i in points]]
                    out_feat.setAttributeMap(atMap)
                    out_feat.setGeometry(outGeom.fromPolygon(polygon))
                    writer.addFeature(out_feat)
                    self.progressBar.setValue(n)
                    n += 1
            elif self.shape == 2:
                while(provider.nextFeature(in_feat)):
                    atMap = in_feat.attributeMap()
                    width = atMap[index1].toDouble()[0]
                    height = atMap[index2].toDouble()[0]
                    angle = atMap[index3].toDouble()[0]      
                    xOffset = float(width/2)
                    yOffset = float(height/2)
                    phi = angle*math.pi/180
                    inGeom = in_feat.geometry()
                    x = inGeom.asPoint().x()
                    y = inGeom.asPoint().y()
                    pt1 = (0., -yOffset)
                    pt2 = (-xOffset, 0.)
                    pt3 = (0., yOffset)
                    pt4 = (xOffset, 0.)
                    points = [pt1, pt2, pt3, pt4]
                    polygon = [[QgsPoint(i[0]*math.cos(phi)+i[1]*math.sin(phi)+x,
                                    -i[0]*math.sin(phi)+i[1]*math.cos(phi)+y) for i in points]]
                    out_feat.setAttributeMap(atMap)
                    out_feat.setGeometry(outGeom.fromPolygon(polygon))
                    writer.addFeature(out_feat)
                    self.progressBar.setValue(n)
                    n += 1
            elif self.shape == 3:
                while(provider.nextFeature(in_feat)):
                    atMap = in_feat.attributeMap()
                    width = atMap[index1].toDouble()[0]
                    height = atMap[index2].toDouble()[0]
                    angle = atMap[index3].toDouble()[0]      
                    xOffset = float(width/2)
                    yOffset = float(height/2)
                    phi = angle*math.pi/180
                    inGeom = in_feat.geometry()
                    x = inGeom.asPoint().x()
                    y = inGeom.asPoint().y()
                    points = []
                    for t in [(2*math.pi)/segments*i for i in range(segments)]:
                          points.append((xOffset*math.cos(t), yOffset*math.sin(t)))
                    polygon = [[QgsPoint(i[0]*math.cos(phi)+i[1]*math.sin(phi)+x,
                                    -i[0]*math.sin(phi)+i[1]*math.cos(phi)+y) for i in points]]
                    out_feat.setAttributeMap(atMap)
                    out_feat.setGeometry(outGeom.fromPolygon(polygon))
                    writer.addFeature(out_feat)
                    self.progressBar.setValue(n)
                    n += 1
        else:
            if self.shape == 1:  
                while(provider.nextFeature(in_feat)):
                    atMap = in_feat.attributeMap()
                    width = atMap[index1].toDouble()[0]
                    height = atMap[index2].toDouble()[0]
                    xOffset = float(width/2)
                    yOffset = float(height/2)
                    inGeom = in_feat.geometry()
                    x = inGeom.asPoint().x()
                    y = inGeom.asPoint().y()
                    pt1 = (-xOffset, -yOffset)
                    pt2 = (-xOffset, yOffset)
                    pt3 = (xOffset, yOffset)
                    pt4 = (xOffset, -yOffset)
                    points = [pt1, pt2, pt3, pt4]
                    polygon = [[QgsPoint(i[0]+x,i[1]+y) for i in points]]
                    out_feat.setAttributeMap(atMap)
                    out_feat.setGeometry(outGeom.fromPolygon(polygon))
                    writer.addFeature(out_feat)
                    self.progressBar.setValue(n)
                    n += 1
            elif self.shape == 2:
                while(provider.nextFeature(in_feat)):
                    atMap = in_feat.attributeMap()
                    width = atMap[index1].toDouble()[0]
                    height = atMap[index2].toDouble()[0]
                    xOffset = float(width/2)
                    yOffset = float(height/2)
                    inGeom = in_feat.geometry()
                    x = inGeom.asPoint().x()
                    y = inGeom.asPoint().y()
                    pt1 = (0., -yOffset)
                    pt2 = (-xOffset, 0.)
                    pt3 = (0., yOffset)
                    pt4 = (xOffset, 0.)
                    points = [pt1, pt2, pt3, pt4]
                    polygon = [[QgsPoint(i[0]+x,i[1]+y) for i in points]]
                    out_feat.setAttributeMap(atMap)
                    out_feat.setGeometry(outGeom.fromPolygon(polygon))
                    writer.addFeature(out_feat)
                    self.progressBar.setValue(n)
                    n += 1
            elif self.shape == 3:
                while(provider.nextFeature(in_feat)):
                    atMap = in_feat.attributeMap()
                    width = atMap[index1].toDouble()[0]
                    height = atMap[index2].toDouble()[0]
                    xOffset = float(width/2)
                    yOffset = float(height/2)
                    inGeom = in_feat.geometry()
                    x = inGeom.asPoint().x()
                    y = inGeom.asPoint().y()
                    points = []
                    for t in [(2*math.pi)/segments*i for i in range(segments)]:
                          points.append((xOffset*math.cos(t), yOffset*math.sin(t)))
                    polygon = [[QgsPoint(i[0]+x,i[1]+y) for i in points]]
                    out_feat.setAttributeMap(atMap)
                    out_feat.setGeometry(outGeom.fromPolygon(polygon))
                    writer.addFeature(out_feat)
                    self.progressBar.setValue(n)
                    n += 1
        del writer
        self.progressBar.setValue(0)
            
                
# Return QgsVectorLayer from a layer name ( as string )
# adopted from 'fTools Plugin', Copyright (C) 2009  Carson Farmer
def getVectorLayerByName( myName ):
	layermap = QgsMapLayerRegistry.instance().mapLayers()
	for name, layer in layermap.iteritems():
		if layer.type() == QgsMapLayer.VectorLayer and layer.name() == myName:
			if layer.isValid():
				return layer
			else:
				return None

# Return list of names of all layers in QgsMapLayerRegistry
# adopted from 'fTools Plugin', Copyright (C) 2009  Carson Farmer
def getLayerNames( vTypes ):
	layermap = QgsMapLayerRegistry.instance().mapLayers()
	layerlist = []
	if vTypes == "all":
		for name, layer in layermap.iteritems():
			layerlist.append( unicode( layer.name() ) )
	else:
		for name, layer in layermap.iteritems():
			if layer.type() == QgsMapLayer.VectorLayer:
				if layer.geometryType() in vTypes:
					layerlist.append( unicode( layer.name() ) )
	return layerlist

# Generate a save file dialog with a dropdown box for choosing encoding style
# adopted from 'fTools Plugin', Copyright (C) 2009  Carson Farmer
def saveDialog( parent ):
	settings = QSettings()
	dirName = settings.value( "/UI/lastShapefileDir" ).toString()
	filtering = QString( "Shapefiles (*.shp)" )
	encode = settings.value( "/UI/encoding" ).toString()
	fileDialog = QgsEncodingFileDialog( parent, "Save output shapefile", dirName, filtering, encode )
	fileDialog.setDefaultSuffix( QString( "shp" ) )
	fileDialog.setFileMode( QFileDialog.AnyFile )
	fileDialog.setAcceptMode( QFileDialog.AcceptSave )
	fileDialog.setConfirmOverwrite( True )
	if not fileDialog.exec_() == QDialog.Accepted:
			return None, None
	files = fileDialog.selectedFiles()
	settings.setValue("/UI/lastShapefileDir", QVariant( QFileInfo( unicode( files.first() ) ).absolutePath() ) )
	return ( unicode( files.first() ), unicode( fileDialog.encoding() ) )

# Return field type from it's name
# adopted from 'fTools Plugin', Copyright (C) 2009  Carson Farmer
def getFieldType(vlayer, fieldName):
	fields = vlayer.dataProvider().fields()
	for name, field in fields.iteritems():
		if field.name() == fieldName:
			return field.typeName()

# Convinience function to add a vector layer to canvas based on input shapefile path ( as string )
# adopted from 'fTools Plugin', Copyright (C) 2009  Carson Farmer
def addShapeToCanvas( shapeFilePath ):
	shapeFilePathList = shapeFilePath.split( "/" )
	layerName = QString( shapeFilePathList[len(shapeFilePathList)-1] )
	if layerName.endsWith( ".shp" ):
		layerName = unicode( layerName ).rstrip( ".shp" )
	vlayer_new = QgsVectorLayer( shapeFilePath, layerName, "ogr" )

	if vlayer_new.isValid():
		QgsMapLayerRegistry.instance().addMapLayer(vlayer_new)
		return True
	else:   
		return False
